import gtk
import os
import sys

# NMR Friend import
import NMRFriend
import NMRFriend.Toolbox.GtkUtil as GtkUtil
from NMRFriend.Toolbox.Depaker import depake

def exit(*args):
    gtk.main_quit()
    sys.exit(0)

def import_spectrum(*args):
    """Open file chooser and import Bruker spectrum"""
    path = GtkUtil.getpathfromfileChooser("Choose Bruker spectrum", "open", ["bruker"])
    if path is not None:
        return import_spectrum_from_path(path)
    else:
        return None

def import_spectrum_from_path(path):
    """Import Bruker spectrum from path"""
    spc = NMRFriend.NmrImport.importBrukerFile(os.path.realpath(path))
    if spc is not None:
        NMRFriend._active_object = spc
        NMRFriend.main_app.update_display()
    return spc

def export_spectrum(*args):
    """Export Bruker spectrum to Ascii file"""
    
    
    spc = NMRFriend._active_object
    if spc is None:
        GtkUtil.message_die("Nothing to be saved!")
        return
    
    path = GtkUtil.getpathfromfileChooser(title="Where do you want me to save exported spectrum?",action="save",filefilter=["all"])
    if path is None:
        return
    
    y = spc.get_real_spc()
    x_pts = spc.get_spc_axis(NMRFriend.UNITS_PTS)
    x_hz = spc.get_spc_axis(NMRFriend.UNITS_HZ)
    x_ppm = spc.get_spc_axis(NMRFriend.UNITS_PPM)
    
    content="#spectrum: %s\nPoints\tHz\tPpm\tY\n"%(spc.title)
    for i in range (len(y)):
        content+="%i\t%f\t%f\t%f\n"%(x_pts[i],x_hz[i],x_ppm[i],y[i])
    
    f=open(path,"w")
    f.write(content)
    f.close()

def export_depaked(*args):
    """Export Depaked spectrum to Ascii file"""
    
    spc = NMRFriend._active_object
    if spc is None:
        GtkUtil.message_die("Nothing to be saved!")
        return
    
    if spc.depaked is None:
        GtkUtil.message_die("Nothing to be saved!")
        return
    
    path = GtkUtil.getpathfromfileChooser(title="Where do you want me to save exported spectrum?",action="save",filefilter=["all"])
    if path is None:
        return


    y = spc.get_depaked()
    x_pts = spc.get_spc_axis(NMRFriend.UNITS_PTS)
    x_hz = spc.get_spc_axis(NMRFriend.UNITS_HZ)
    x_ppm = spc.get_spc_axis(NMRFriend.UNITS_PPM)
    
    content="#spectrum: De-paked from %s\nPoints\tHz\tPpm\tY\n"%(spc.title)
    for i in range (len(y)):
        content+="%i\t%f\t%f\t%f\n"%(x_pts[i],x_hz[i],x_ppm[i],y[i])
    
    f=open(path,"w")
    f.write(content)
    f.close()

def show_about(*args):
    """Display information about NMR Friend"""
    response = NMRFriend.main_app.about.run()
    
    NMRFriend.main_app.about.hide()

def calculate_moments(*args):
    """Calculate Spectrum Moments"""
    if NMRFriend._active_object is None:
        GtkUtil.message_die("No active spectrum!", NMRFriend.MESSAGE_AVERT)
        return
    
    moments = NMRFriend._active_object.moments
    limits = NMRFriend._active_object.limits
    
    if limits is None:
        GtkUtil.message_die("You must define limits BEFORE calculating moments")
        return
    
    if moments is None:
        NMRFriend._active_object.moments = moments = NMRFriend.NmrSpectrum.Moments(NMRFriend._active_object)
    
    # Load Waiting window
    waiter = GtkUtil.Waiting_window(NMRFriend.main_app, text = "Calculating moments...")
    waiter.start_pulsing()

    # Actually calculate moments
    moments.calc_moments()
    
    # We are done!
    waiter.task_done()

def depake_spectrum(*args):
    """Depake Spectrum"""
    limits = NMRFriend._active_object.limits
    
    if NMRFriend._active_object is None:
        GtkUtil.message_die("No active spectrum!", NMRFriend.MESSAGE_AVERT)
        return
    
    if limits is None:
        GtkUtil.message_die("You must define limits BEFORE Depaking!")
        return
    
    # Get limits and pivot
    llim, rlim = NMRFriend._active_object.limits.get_limits()
    n0 = NMRFriend._active_object.limits.get_pivot()
    
    # perform baseline correction
    sum = 0.0
    spc = NMRFriend._active_object.get_real_spc()
    ncp = len(spc)
    for i in range(int(llim)):
        sum += spc[i]
    
    for i in range(int(rlim-1),ncp):
        sum += spc[i]
    
    sums = sum/(ncp - rlim + llim + 1)
    
    for i in range(len(spc)):
        spc[i] = spc[i] - sums
    
    # Load Waiting window
    waiter = GtkUtil.Waiting_window(NMRFriend.main_app, text = "DePaking spectrum...")
    waiter.start_pulsing()
    
    # Do the De-Pake-ing   
    oriented_spc = depake(spc, llim, rlim, n0, n_iter = 5)
    
    NMRFriend._active_object.set_depaked(oriented_spc)
    
    # We are done!
    waiter.task_done()
    
def remove_status(*args):
  """Remove remove from status bar"""
  NMRFriend.main_app.statusbar.remove_status()