import os,string,struct

# NMR Friend import
import NMRFriend
import NMRFriend.Toolbox.GtkUtil as GtkUtil
import NMRFriend.Toolbox.Util as Util

def readBinaryFile(filename, is_bigE=0):
  """Read a binary file and extract a list containing number from file (format:long). Returns list if OK, empty list object if not."""
  try:
    f = open(filename,"rb")
    content = f.read()
    f.close()
    
    data=list()
    
    for i in range(0,len(content),4):
        if is_bigE == 1:
            data.append(struct.unpack(">l",content[i:i+4])[0])
        else:
            data.append(struct.unpack("<l",content[i:i+4])[0])
    return data
  except IOError:
    #gtkutil.message_die("Can't open Spectrum or FID file",gtkutil.MESSAGE_AVERT)
    return []

def readBrukerParam(filepath,paramlist=[]):
  """Read Bruker Parameter file and search for a list of parameter. Return a dictionnary with parameter names as keys and parameter values as values (values = None object if parameter)."""
  try :
    result={}
    f=open(filepath,'r')
  except IOError:
    pass
  #GtkUtil.message_die("Can't open Parameter file",GtkUtil.MESSAGE_AVERT)
    #return None
  else:
    for line in f:
    # Parameter and value extraction from line
      if line.find("##$") != -1: # this is a parameter line
        tmpparam=line[line.find("##$")+3:line.find("=")]
        tmpvalue=line[line.find("= ")+2:]
      
        if tmpparam in paramlist:  # check if extracted parameter is useful
          result[tmpparam]=Util.str2num(tmpvalue)
    f.close()
  #complete result with unfound parameters
  for param in paramlist:
    if result.has_key(param)==False:
      result[param]=None
  return result

def importBrukerFile(path):
  """Imports a bruker file (FID + processed spectrum). Return SpcBruker object or None object if something bad happens"""
  # Initialization
  need_to_be_processed=False
  
  #print "importing"
  
  # Check if path really exists
  if os.path.exists(path)==False:
    GtkUtil.message_die("The path given is not correct!\n'%s' does not exist!"%path,Toolbox.MESSAGE_INFO)
    return None
  
  #Find root directory of spectrum
  parent=os.path.split(path)[0]
  child=os.path.split(path)[1]
  if os.path.isfile(path):
    path=parent
  if 'pdata' in os.listdir(path):
    rootpath=path
  elif string.rfind(path,"pdata") !=-1:
    rootpath=path[0:string.rfind(path,"pdata")]
  else:
    rootpath=path
  expname=os.path.split(os.path.split(rootpath)[0])[1]
  renum=os.path.split(os.path.split(rootpath)[1])[1]
  
  #Yeah got root directory of spectrum. Now Easy going part: get parameters
  # 1st: get acquisition parameters
  acqparam=["AQSEQ","BF1","BYTORDA","NS","NUC1","NC","O1","OVERFLW","SFO1","SW","SW_h","TD","TE"]
  acqparam=readBrukerParam(os.path.join(rootpath,"acqus"),acqparam)
  #print acqparam
  
  # 2nd: get FID
  fid=readBinaryFile(os.path.join(rootpath,"fid"),acqparam["BYTORDA"])
  
  #Return None if no FID found... Don't import spectrum if no FID
  if len(fid)==0:
    return None
  
  # If still there, FID is found
  # 3rd: get processing parameters and title... for every processed spectra
  # same time 4th: build spectra object
  # Hard part
  tmpSpectra=[]
  if os.path.exists(os.path.join(rootpath,"pdata")):
    for procpath in os.listdir(os.path.join(rootpath,"pdata")):
      if len(procpath.strip("1234567890")) != 0:
        print "not a valid processing directory!"
        continue
      
      #Get processing parameters
      procparam=readBrukerParam(os.path.join(rootpath,"pdata",procpath,"procs"),["BYTORDP","FTSIZE","LB","NC_proc","NSP","OFFSET","PHC0","PHC1","SI","SW_p","SF"])
      
      #Get Real Spectrum
      reSpc=readBinaryFile(os.path.join(rootpath,"pdata",procpath,"1r"),procparam["BYTORDP"])
      
      #Get Imaginary Spectrum
      imSpc=readBinaryFile(os.path.join(rootpath,"pdata",procpath,"1i"),procparam["BYTORDP"])
      
      #Get title
      try :
        f=open(os.path.join(rootpath,"pdata",procpath,"title"),'r')
      except IOError:
        title=""
      else:
        title=f.read().strip()
        title=unicode(title,"iso8859-15")
        title=title.encode("utf-8")
        f.close()
      
      # build Spectrum object
      tmpSpc=NMRFriend.NmrSpectrum.SpcBruker()
      
      # Sets acquisition parameters
      for paramname in acqparam:
        spcparam=paramname.lower()
        if spcparam in tmpSpc.get_params_names():
          tmpSpc.set_param_value(paramname.lower(),acqparam[paramname])
      
      # Same for processing parameters
      for paramname in procparam:
        spcparam=paramname.lower()
        if spcparam in tmpSpc.get_params_names():
          tmpSpc.set_param_value(paramname.lower(),procparam[paramname])
      
      # Special treatment for sw_hz, sw_ppm, o1_hz and o1_ppm
      tmpSpc.set_param_value("sw_hz",acqparam["SW_h"])
      tmpSpc.set_param_value("sw_ppm",acqparam["SW"])
      tmpSpc.set_param_value("o1_hz",acqparam["O1"])
      tmpSpc.set_param_value("o1_ppm",float(acqparam["O1"])/float(acqparam["BF1"]))
      
      # Special treatment for srs...
      tmpSpc.set_param_value("sr_hz",procparam["SF"]-(acqparam["SFO1"]-acqparam["O1"]*0.000001))
      tmpSpc.set_param_value("sr_ppm",(procparam["SF"]-(acqparam["SFO1"]-acqparam["O1"]*0.000001))/float(acqparam["BF1"]))
      
      # Sets Title
      tmpSpc.set_title(title)
      
      # Need to multiply spectrum by 2 factor
      factor=2**float(procparam["NC_proc"])
      
      for i in range(len(reSpc)):
        reSpc[i]=reSpc[i]*factor
        imSpc[i]=imSpc[i]*factor
      
      # Stores spectra and FID
      tmpSpc.set_fid(list(fid))
      tmpSpc.set_spc(list(reSpc),list(imSpc))
      
      # Computes increments
      tmpSpc.set_param_value("inc_tps",float(1)/acqparam["SW_h"])
      tmpSpc.set_param_value("inc_ppm",float(acqparam["SW"])/(procparam["SI"]-1))
      tmpSpc.set_param_value("inc_hz",float(acqparam["SW_h"])/(procparam["SI"]-1))
      
      tmpSpc.processed=True
      
      tmpSpectra.append([expname,renum,procpath,tmpSpc])
  else:
    pass
  
  # If no processed spectra found, need to create a spectrum object to display FID
  if len(tmpSpectra)==0:
    
    tmpSpc=NMRFriend.NmrSpectrum.SpcBruker()
    
    # Sets acquisition parameters
    for paramname in acqparam:
      spcparam=paramname.lower()
      if spcparam in tmpSpc.get_params_names():
        tmpSpc.set_param_value(paramname.lower(),acqparam[paramname])
    
    # Special treatment for sw_hz, sw_ppm, o1_hz and o1_ppm
    tmpSpc.set_param_value("sw_hz",acqparam["SW_h"])
    tmpSpc.set_param_value("sw_ppm",acqparam["SW"])
    tmpSpc.set_param_value("o1_hz",acqparam["O1"])
    tmpSpc.set_param_value("o1_ppm",float(acqparam["O1"])/float(acqparam["BF1"]))
    
    # Sets Title
    tmpSpc.set_title("")
    
    # Stores spectra and FID
    tmpSpc.set_fid(list(fid))
    tmpSpc.set_spc([],[])
    
    # Computes increments
    tmpSpc.set_param_value("inc_tps",float(1)/acqparam["SW_h"])
    
    tmpSpc.processed=False
  
  return tmpSpc