import gtk,os,gobject
from matplotlib.cbook import Stack

#NMR Friend Imports
import NMRFriend
import NMRFriend.Toolbox.GtkUtil as GtkUtil

class Cursors:  #namespace
    HAND, POINTER, SELECT_REGION, MOVE = range(4)
cursors = Cursors()

cursord = {
    cursors.MOVE          : gtk.gdk.Cursor(gtk.gdk.FLEUR),
    cursors.HAND          : gtk.gdk.Cursor(gtk.gdk.HAND2),
    cursors.POINTER       : gtk.gdk.Cursor(gtk.gdk.LEFT_PTR),
    cursors.SELECT_REGION : gtk.gdk.Cursor(gtk.gdk.TCROSS),
    }

class NMRFriend_Navigation_Toolbar(gtk.Toolbar):
    """Base class for the navigation cursor.
    Implemented using matplotlib toolbar 2.
    """
    # list of toolitems to add to the toolbar, format is:
    # text, tooltip_text, image_file, callback(str),button type,button group
    toolitems = (
        ('Home', 'Reset original view', 'view_home.png', 'home',NMRFriend.TOOLBAR_BUTTON,None),
        ('Back', 'Back to  previous view','view_previous.png', 'back',NMRFriend.TOOLBAR_BUTTON,None),
        ('Forward', 'Forward to next view','view_next.png', 'forward',NMRFriend.TOOLBAR_BUTTON,None),
        (None, None, None, None, None,None),
        ('NMRZoom','Zoom to region','nmrzoom.png','nmrzoom',NMRFriend.TOOLBAR_RADIO_BUTTON,"Interaction"),
        ('Measure','Measure tool','measure.png','measure',NMRFriend.TOOLBAR_RADIO_BUTTON,"Interaction"),
        ('Pan', 'Pan axes with left mouse, zoom with right', 'pan.png','pan',NMRFriend.TOOLBAR_RADIO_BUTTON,"Interaction"),
        ('Zoom', 'Zoom to rectangle','zoom_rect.png', 'zoom',NMRFriend.TOOLBAR_RADIO_BUTTON,"Interaction"),
        (None, None, None, None, None,None),
        ('ZoomIn', 'Zoom in','zoom_in.png', 'zoom_in',NMRFriend.TOOLBAR_BUTTON,None),
        ('ZoomOut', 'Zoom out','zoom_out.png', 'zoom_out',NMRFriend.TOOLBAR_BUTTON,None),
        ('Rescale', 'Rescale','rescale.png', 'rescale',NMRFriend.TOOLBAR_BUTTON,None),
        ('RescaleX', 'Rescale X axis','rescale-x.png', 'rescaleX',NMRFriend.TOOLBAR_BUTTON,None),
        ('RescaleY', 'Rescale Y axis','rescale-y.png', 'rescaleY',NMRFriend.TOOLBAR_BUTTON,None),
        (None, None, None, None, None,None),
        )
    
    def __init__(self, canvas, window):
        self.win = window
        gtk.Toolbar.__init__(self)
        
        self._idleId = 0
        self.canvas = canvas
        
        # a dict from axes index to a list of view limits
        self._views = Stack()
        self._xypress = None  # the  location and axis info at the time of the press 
        
        if self.canvas.needs_toolbar():
            self._idPress = self.canvas.mpl_connect('button_press_event', self.press)
            self._idRelease = self.canvas.mpl_connect('button_release_event', self.release)
        else:
            self._idPress=None
            self._idRelease=None
        
        self._active = NMRFriend.NMRGRAPH_INTERACTION_ZOOM_NMR
        self._lastCursor = None
        
        # Pango layout to write text on figure
        self.pangolayout = self.canvas.create_pango_layout("")
        
        # initialize widgets
        self._init_toolbar()
        
        # Connect Signals
        self._idDrag=self.canvas.mpl_connect('motion_notify_event', self.mouse_move)  
        
        self._button_pressed = None # determined by the button pressed at start
        self.mode = ''  # a mode string for the status bar
        
        self.set_history_buttons()
    
    def _init_toolbar(self):
        self.set_style(gtk.TOOLBAR_ICONS)
        self.tooltips = gtk.Tooltips()

        button_groups={}
        
        for text, tooltip_text, image_file, callback,button_type,button_group in self.toolitems:
            if text == None:
                self.insert( gtk.SeparatorToolItem(), -1 )
                continue
            fname = os.path.join(NMRFriend.PixmapDir, image_file)
            image = gtk.Image()
            image.set_from_file(fname)
            if button_type==NMRFriend.TOOLBAR_BUTTON:
                tbutton = gtk.ToolButton(image, text)
            elif button_type==NMRFriend.TOOLBAR_TOGGLE_BUTTON:
                tbutton = gtk.ToggleToolButton()
                tbutton.set_icon_widget(image)
            elif button_type==NMRFriend.TOOLBAR_RADIO_BUTTON:
                if button_groups.has_key(button_group):
                    tbutton = gtk.RadioToolButton(button_groups[button_group])
                else:
                    tbutton = gtk.RadioToolButton()
                    button_groups[button_group]=tbutton
                tbutton.set_icon_widget(image)
            self.insert(tbutton, -1)
            if self.canvas.needs_toolbar():
                tbutton.connect('clicked', getattr(self, callback))
            tbutton.set_tooltip(self.tooltips, tooltip_text, 'Private')
    
        toolitem = gtk.SeparatorToolItem()
        self.insert(toolitem, -1)
        # set_draw() not making separator invisible,
        # bug #143692 fixed Jun 06 2004, will be in GTK+ 2.6
        toolitem.set_draw(False)
        toolitem.set_expand(True)
    
        toolitem = gtk.ToolItem()
        self.insert(toolitem, -1)
        
    
        if self.canvas.needs_toolbar():
            self.show_all()
    
    def home(self, *args):
        'restore the original view'
        self._views.home()
        self.set_history_buttons()
        self._update_view()
    
    def forward(self, *args):
        'move forward in the view lim stack'
        self._views.forward()
        self.set_history_buttons()
        self._update_view()
    
    def back(self, *args):
        'move back up the view lim stack'
        self._views.back()
        self.set_history_buttons()
        self._update_view()
    
    def set_history_buttons(self):
        'enable or disable back/forward button'
        pass
    
    def push_current(self):
        'push the current view limits onto the stack'
        lims = []
        for a in self.canvas.figure.get_axes():
            xmin, xmax = a.get_xlim()
            ymin, ymax = a.get_ylim()
            lims.append( (xmin, xmax, ymin, ymax) )
        self._views.push(lims)
        self.set_history_buttons()
    
    def empty_views(self):
        """  Delete stocked views & push current."""
        self._views.clear()
    
    
    def _update_view(self):
        'update the viewlim from the view stack for each axes'

        lims = self._views()
        if lims is None:  return
        for i, a in enumerate(self.canvas.figure.get_axes()):
            xmin, xmax, ymin, ymax = lims[i]
            a.set_xlim((xmin, xmax))
            a.set_ylim((ymin, ymax))

        self.draw()
    
    def press(self, event):
        'this will be called whenever a mouse button is pressed'
        
        if self._active==NMRFriend.NMRGRAPH_INTERACTION_ZOOM_NMR:
            self.press_nmrzoom(event)
            
        elif self._active==NMRFriend.NMRGRAPH_INTERACTION_MEASURE:
            self.press_nmrzoom(event)
            
        elif self._active==NMRFriend.NMRGRAPH_INTERACTION_PAN:
            self.press_pan(event)
            
        elif self._active==NMRFriend.NMRGRAPH_INTERACTION_ZOOM_RECT:
            self.press_zoom(event)
    
    def release(self, event):
        try: del self._imageBack
        except AttributeError: pass
        
        if self._active==NMRFriend.NMRGRAPH_INTERACTION_ZOOM_NMR:
            self.release_nmrzoom(event)
            
        elif self._active==NMRFriend.NMRGRAPH_INTERACTION_MEASURE:
            self.release_nmrzoom(event)
            
        elif self._active==NMRFriend.NMRGRAPH_INTERACTION_PAN:
            self.release_pan(event)
            
        elif self._active==NMRFriend.NMRGRAPH_INTERACTION_ZOOM_RECT:
            self.release_zoom(event)

    def mouse_move(self, event):
        
        if len(self.canvas.axes)==0:
            return
        
        if not event.inaxes or self._active==NMRFriend.NMRGRAPH_INTERACTION_NONE:
            if self._lastCursor != cursors.POINTER:
                self.set_cursor(cursors.POINTER)
                self._lastCursor = cursors.POINTER
        else:
            if self._active==NMRFriend.NMRGRAPH_INTERACTION_ZOOM_RECT:
                if self._lastCursor != cursors.SELECT_REGION:
                    self.set_cursor(cursors.SELECT_REGION)
                    self._lastCursor = cursors.SELECT_REGION
                if self._xypress is not None:
                    x, y = event.x, event.y
                    lastx, lasty, a, ind, lim, trans= self._xypress
                    self.draw_rubberband(event, x, y, lastx, lasty)
                
            elif self._active==NMRFriend.NMRGRAPH_INTERACTION_ZOOM_NMR or self._active==NMRFriend.NMRGRAPH_INTERACTION_MEASURE:
                if self._lastCursor != cursors.POINTER:
                    self.set_cursor(cursors.POINTER)
                    self._lastCursor = cursors.POINTER
                if self._xypress is not None:
                    x, y = event.x, event.y
                    lastx, lasty, a, ind, lim, trans= self._xypress
                    self.draw_nmrrubberband(event,x,lastx)
                
            elif (self._active==NMRFriend.NMRGRAPH_INTERACTION_PAN and
                    self._lastCursor != cursors.MOVE):
                self.set_cursor(cursors.MOVE)
                
                self._lastCursor = cursors.MOVE
 
    
    def set_cursor(self, cursor):
        self.canvas.window.set_cursor(cursord[cursor])

    def pan(self,*args):
        """Activate the pan/zoom tool. pan with left button, zoom with right"""
        self._active = NMRFriend.NMRGRAPH_INTERACTION_PAN
        


    def press_pan(self, event):
        'the press mouse button in pan/zoom mode callback'
        #print "in press _pan"
        if event.button == 1:
            self._button_pressed=1
        elif  event.button == 3: 
            self._button_pressed=3
        else: 
            self._button_pressed=None
            return

        x, y = event.x, event.y

        # push the current view to define home if stack is empty
        if self._views.empty(): self.push_current()
        
            
        for i, a in enumerate(self.canvas.figure.get_axes()):
            if event.inaxes == a and event.inaxes.get_navigate():
                xmin, xmax = a.get_xlim()
                ymin, ymax = a.get_ylim()
                lim = xmin, xmax, ymin, ymax
                self._xypress = x, y, a, i, lim,a.transData.deepcopy()
                self.canvas.mpl_disconnect(self._idDrag)   
                self._idDrag=self.canvas.mpl_connect('motion_notify_event', self.drag_pan) 
                break

    
    def drag_pan(self, event):
        'the drag callback in pan/zoom mode'
        def format_deltas(event,dx,dy):
            if event.key=='control':
                if(abs(dx)>abs(dy)):
                    dy = dx
                else:
                    dx = dy
            elif event.key=='x':
                dy = 0
            elif event.key=='y':
                dx = 0
            elif event.key=='shift':
                if 2*abs(dx) < abs(dy):
                    dx=0
                elif 2*abs(dy) < abs(dx):
                    dy=0
                elif(abs(dx)>abs(dy)):
                    dy=dy/abs(dy)*abs(dx)
                else:    
                    dx=dx/abs(dx)*abs(dy)
            return (dx,dy)                
                    
        if self._xypress is None: return
        x, y = event.x, event.y

        lastx, lasty, a, ind, lim, trans = self._xypress
        xmin, xmax, ymin, ymax = lim
        #safer to use the recorded button at the press than current button: 
        #multiple button can get pressed during motion...   
        if self._button_pressed==1: 
            lastx, lasty = trans.inverse_xy_tup( (lastx, lasty) )
            x, y = trans.inverse_xy_tup( (x, y) )
            if a.get_xscale()=='log':
                dx=1-lastx/x
            else:    
                dx=x-lastx
            if a.get_yscale()=='log':
                dy=1-lasty/y
            else:    
                dy=y-lasty
            dx,dy=format_deltas(event,dx,dy)
            if a.get_xscale()=='log':
                xmin *= 1-dx
                xmax *= 1-dx
            else: 
                xmin -= dx
                xmax -= dx   
            if a.get_yscale()=='log':
                ymin *= 1-dy
                ymax *= 1-dy
            else:
                ymin -= dy
                ymax -= dy  
        
        
        # Trick for not get too much left or right
        Xmin,Xmax=self.canvas.get_x_lims()
        if xmin<Xmin:
            xmax=Xmin+(xmax-xmin)
            xmin=Xmin
        if xmax>Xmax:
            xmin=Xmax-(xmax-xmin)
            xmax=Xmax
        
        a.set_xlim((xmin, xmax))
        a.set_ylim((ymin, ymax))
        
        self.dynamic_update()
    
    def release_pan(self, event):
        'the release mouse button callback in pan/zoom mode'
        self.canvas.mpl_disconnect(self._idDrag)   
        self._idDrag=self.canvas.mpl_connect('motion_notify_event', self.mouse_move)   
        if self._xypress is None: return
        self._xypress = None
        self._button_pressed=None
        self.push_current()
        self.draw()
        
    
    def rescale(self,*args):
        """Rescale Plots"""
        self.canvas.rescale()
    
    def rescaleX(self,*args):
        """Rescale X axis"""
        self.canvas.rescaleX()
    
    def rescaleY(self,*args):
        """Rescale Y axis"""
        self.canvas.rescaleY()
    
    def zoom_in(self,*args):
        """Zoom in (Y axis)."""
        # push the current view to define home if stack is empty
        if self._views.empty(): self.push_current()
        
        for i, a in enumerate(self.canvas.figure.get_axes()):
            ymin, ymax = a.get_ylim()
            delta=ymax-ymin
            #ymin=ymin+.25*delta
            ymax=ymax-0.2*delta
            a.set_ylim(ymin,ymax)
        
        
        self.draw()
        self.push_current()

    
    def zoom_out(self,*args):
        """Zoom out (Y axis)."""
        # push the current view to define home if stack is empty
        if self._views.empty(): self.push_current()
        
        for i, a in enumerate(self.canvas.figure.get_axes()):
            ymin, ymax = a.get_ylim()
            delta=ymax-ymin
            #ymin=ymin+.25*delta
            ymax=ymax+0.2*delta
            a.set_ylim(ymin,ymax)
        
        
        self.draw()
        self.push_current()

    def zoom(self, *args):
        'activate zoom to rect mode'
        self._active = NMRFriend.NMRGRAPH_INTERACTION_ZOOM_RECT
        


    
    def press_zoom(self, event):
        'the press mouse button in zoom to rect mode callback'
        if event.button == 1:
            self._button_pressed=1
        elif  event.button == 3: # right click
            self._button_pressed=3
            
        else: 
            self._button_pressed=None
            return
            
        x, y = event.x, event.y

        # push the current view to define home if stack is empty
        if self._views.empty(): self.push_current()

        for i, a in enumerate(self.canvas.figure.get_axes()):
            if event.inaxes==a and event.inaxes.get_navigate():
                xmin, xmax = a.get_xlim()
                ymin, ymax = a.get_ylim()
                lim = xmin, xmax, ymin, ymax
                self._xypress = x, y, a, i, lim, a.transData.deepcopy()

                break


    def nmrzoom(self, *args):
        'activate zoom to spectrum region mode'
        self._active = NMRFriend.NMRGRAPH_INTERACTION_ZOOM_NMR
        
    
    def measure(self, *args):
        'activate measure mode'
        self._active = NMRFriend.NMRGRAPH_INTERACTION_MEASURE
        

    
    def press_nmrzoom(self, event):
        'the press mouse button in zoom to spectrum region mode callback'
        if event.button == 1:
            self._button_pressed=1
        else: 
            self._button_pressed=None
            return
        
        x, y = event.x, event.y

        # push the current view to define home if stack is empty
        if self._views.empty(): self.push_current()

        for i, a in enumerate(self.canvas.figure.get_axes()):
            if event.inaxes==a and event.inaxes.get_navigate():
                xmin, xmax = a.get_xlim()
                ymin, ymax = a.get_ylim()
                lim = xmin, xmax, ymin, ymax
                self._xypress = x, y, a, i, lim, a.transData.deepcopy()
                break
    
    def release_nmrzoom(self, event):
        'the release mouse button callback in zoom to spectrum region mode'        
        if self._xypress is None:
            return
        
        x, y = event.x, event.y

        
        lastx, lasty, a, ind, lim, trans = self._xypress
        # ignore singular clicks - 5 pixels is a threshold
        if abs(x-lastx)<5 or self._active==NMRFriend.NMRGRAPH_INTERACTION_MEASURE:
            self._xypress = None
            self.draw()
            return

        xmin, ymin, xmax, ymax = lim

        # zoom to spectrum region
        lastx, lasty = a.transData.inverse_xy_tup( (lastx, lasty) )
        x, y = a.transData.inverse_xy_tup( (x, y) )  
        Xmin,Xmax=a.get_xlim()
        Ymin,Ymax=a.get_ylim()          

        if Xmin < Xmax:
            if x<lastx:  xmin, xmax = x, lastx
            else: xmin, xmax = lastx, x  
            if xmin < Xmin: xmin=Xmin
            if xmax > Xmax: xmax=Xmax
        else: 
            if x>lastx:  xmin, xmax = x, lastx
            else: xmin, xmax = lastx, x  
            if xmin > Xmin: xmin=Xmin
            if xmax < Xmax: xmax=Xmax
        if self._button_pressed == 1:  
            a.set_xlim((xmin, xmax))
            a.set_ylim((Ymin, Ymax))
        
        self.draw()
        self._xypress = None
        self._button_pressed == None

        self.push_current()
    
    
    def draw_rubberband(self, event, x0, y0, x1, y1):
        'adapted from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/189744'
        drawable = self.canvas.window
        if drawable == None: return

        gc = drawable.new_gc()

        height = self.canvas.figure.bbox.height()
        y1 = height - y1
        y0 = height - y0
        
        w = abs(x1 - x0)
        h = abs(y1 - y0)

        rect = [int(val)for val in min(x0,x1), min(y0, y1), w, h]
        try: lastrect, imageBack = self._imageBack
        except AttributeError:
            #snap image back        
            if event.inaxes == None: return

            ax = event.inaxes
            l,b,w,h = [int(val) for val in self.canvas.get_figure_bounds()]
            
            axrect = l,b,w,h
            self._imageBack = axrect, drawable.get_image(*axrect)            
            drawable.draw_rectangle(gc, False, *rect)
            self._idleId = 0
        else:
            def idle_draw(*args):

                drawable.draw_image(gc, imageBack, 0, 0, *lastrect)
                drawable.draw_rectangle(gc, False, *rect)
                self._idleId = 0
                return False
            if self._idleId==0:
                self._idleId = gobject.idle_add(idle_draw)
    
    
    
    def draw_nmrrubberband(self,event,x0,x1):
        'adapted from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/189744'
        drawable = self.canvas.window
        
        #print drawable
        #add color to colormap
        color=drawable.get_colormap().alloc_color("#88DDFF")
       
        
        if drawable == None: return
        gc = drawable.new_gc()
        #change color for gc
        gc.set_foreground(color)
        #Set line width
        gc.line_width=2
        
      
        try: lastrect, imageBack = self._imageBack
        except AttributeError:
            #snap image back        
            if event.inaxes == None: return

            ax = event.inaxes
            l,b,w,h = [int(val) for val in self.canvas.get_figure_bounds()]
            axrect = l,b,w,h
            self._imageBack = axrect, drawable.get_image(*axrect)
            self.do_draw_nmrrubberband(event,drawable,gc,x0,x1)
            self._idleId = 0
        else:
            def idle_draw(*args):
                
                drawable.draw_image(gc, imageBack, 0, 0, *lastrect)
                self.do_draw_nmrrubberband(event,drawable,gc,x0,x1)
                self._idleId = 0
                return False
            if self._idleId==0:
                self._idleId = gobject.idle_add(idle_draw)
    
    def do_draw_nmrrubberband(self,event,drawable,gc,X0,X1):
        """Actually draws NMR interaction 'rubber band'.
        Pure GTK stuff for velocity.
        """
        
        x0=int(min(X0,X1))
        x1=int(max(X0,X1))
        xmin,ymin,width,height=[int(val) for val in event.inaxes.bbox.get_bounds()]
        
        # ymin adjustement
        ymin+=2
        
        lline=x0,ymin,x0,ymin+height-1 #left line
        rline=x1,ymin,x1,ymin+height-1 #right line
        
        # median Y
        ymed=int(ymin+height/2)
        
        link_line=x0,ymed,x1,ymed # linking line
        
        #Size of arrows
        size=4
        larrow=((x0,ymed),(x0+size,ymed-size),(x0+size,ymed+size))
        rarrow=((x1,ymed),(x1-size,ymed-size),(x1-size,ymed+size))
        
        # draw stuff
        drawable.draw_line(gc, *lline)
        drawable.draw_line(gc, *rline)
        drawable.draw_line(gc, *link_line)
        drawable.draw_polygon(gc,True,larrow)
        drawable.draw_polygon(gc,True,rarrow)
        
        # add text
        x0tr,y0tr=event.inaxes.transData.inverse_xy_tup( (X0, 0) )
        x1tr,y1tr=event.inaxes.transData.inverse_xy_tup( (X1, 0) )
        
        if self._active==NMRFriend.NMRGRAPH_INTERACTION_ZOOM_NMR:
            delta=abs(x0tr-x1tr)
        else:
            delta=x0tr-x1tr
        
        if self.canvas.get_units()=="Points":
            delta=int(delta)
        
        self.pangolayout.set_text(str(delta)+" "+self.canvas.get_units())
        layout_width,layout_height=self.pangolayout.get_pixel_size()
        drawable.draw_layout(gc, (x1+x0-layout_width)/2, int(ymed-5-layout_height), self.pangolayout,gtk.gdk.color_parse("#3388FF"),None)
    
    def release_zoom(self, event):
        'the release mouse button callback in zoom to rect mode'        
        if self._xypress is None: return
        x, y = event.x, event.y
        
        lastx, lasty, a, ind, lim, trans = self._xypress
        # ignore singular clicks - 5 pixels is a threshold
        if abs(x-lastx)<5 or abs(y-lasty)<5:
            self._xypress = None
            self.draw()
            return

        xmin, ymin, xmax, ymax = lim

        # zoom to rect
        lastx, lasty = a.transData.inverse_xy_tup( (lastx, lasty) )
        x, y = a.transData.inverse_xy_tup( (x, y) )  
        Xmin,Xmax=a.get_xlim()
        Ymin,Ymax=a.get_ylim()          

        if Xmin < Xmax:
            if x<lastx:  xmin, xmax = x, lastx
            else: xmin, xmax = lastx, x  
            if xmin < Xmin: xmin=Xmin
            if xmax > Xmax: xmax=Xmax
        else: 
            if x>lastx:  xmin, xmax = x, lastx
            else: xmin, xmax = lastx, x  
            if xmin > Xmin: xmin=Xmin
            if xmax < Xmax: xmax=Xmax
                
        if Ymin < Ymax:
            if y<lasty:  ymin, ymax = y, lasty
            else: ymin, ymax = lasty, y  
            if ymin < Ymin: ymin=Ymin
            if ymax > Ymax: ymax=Ymax
        else: 
            if y>lasty:  ymin, ymax = y, lasty
            else: ymin, ymax = lasty, y  
            if ymin > Ymin: ymin=Ymin
            if ymax < Ymax: ymax=Ymax

        if self._button_pressed == 1:  
            a.set_xlim((xmin, xmax))
            a.set_ylim((ymin, ymax))
        elif self._button_pressed == 3: 
            if a.get_xscale()=='log':
                alpha=log(Xmax/Xmin)/log(xmax/xmin)
                x1=pow(Xmin/xmin,alpha)*Xmin
                x2=pow(Xmax/xmin,alpha)*Xmin
            else:    
                alpha=(Xmax-Xmin)/(xmax-xmin)
                x1=alpha*(Xmin-xmin)+Xmin
                x2=alpha*(Xmax-xmin)+Xmin
            if a.get_yscale()=='log':
                alpha=log(Ymax/Ymin)/log(ymax/ymin)
                y1=pow(Ymin/ymin,alpha)*Ymin
                y2=pow(Ymax/ymin,alpha)*Ymin
            else:    
                alpha=(Ymax-Ymin)/(ymax-ymin)
                y1=alpha*(Ymin-ymin)+Ymin
                y2=alpha*(Ymax-ymin)+Ymin
            a.set_xlim((x1, x2))
            a.set_ylim((y1, y2))    

        self.draw()
        self._xypress = None
        self._button_pressed == None

        self.push_current()


    
    def draw(self):
        'redraw the canvases, update the locators'
        for a in self.canvas.figure.get_axes():
            xaxis = getattr(a, 'xaxis', None)
            yaxis = getattr(a, 'yaxis', None)
            locators = []
            if xaxis is not None:
                locators.append(xaxis.get_major_locator())
                locators.append(xaxis.get_minor_locator())                
            if yaxis is not None:
                locators.append(yaxis.get_major_locator())
                locators.append(yaxis.get_minor_locator())                

            for loc in locators:
                loc.refresh()
        self.canvas.draw()
    
    def dynamic_update(self):
        # legacy method; new method is canvas.draw_idle
        self.canvas.draw_idle()


class Spc_Navigation_Toolbar(NMRFriend_Navigation_Toolbar):
    # list of toolitems to add to the toolbar, format is:
    # text, tooltip_text, image_file, callback(str),button type,button group
    
    toolitems = NMRFriend_Navigation_Toolbar.toolitems + (('ToggleUnits', 'Toggle between Points, PPM and Hz units', 'units-spc.png', 'toggle_units',NMRFriend.TOOLBAR_BUTTON,None),
                                                          (None,None,None,None,None,None),
                                                          ('SetLims', 'Set limits and pivot to be used for moments/DePaking calculation.', 'lim.png', 'set_lims',NMRFriend.TOOLBAR_BUTTON,None),
                                                          (None,None,None,None,None,None),
                                                          ('CalcMoments', 'Calculate spectrum moments', 'moments-calc.png', 'moments_calc',NMRFriend.TOOLBAR_BUTTON,None),
                                                          ('DepakeSpc', 'Depake spectrum', 'depake.png', 'depake',NMRFriend.TOOLBAR_BUTTON,None),
                                                          )
    
    def __init__(self, canvas, window):
        NMRFriend_Navigation_Toolbar.__init__(self,canvas,window)
        
        self._last_active=None
        self._movable_line=None
        
    def set_lims(self,*args):
        limits = self.canvas.spectrum.limits
        
        if limits is None:
            self.canvas.spectrum.limits = limits = NMRFriend.NmrSpectrum.Limits(self.canvas.spectrum)
        
        axis=self.canvas.figure.get_axes()[0]
        
        xmin,xmax=axis.get_xlim()
        mid=(xmax+xmin)/2.0
        delta=(abs(xmax)+abs(xmin))*0.25
        
        # Reset limits
        lim1_tr=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,mid-delta)
        lim2_tr=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,mid+delta)
        limits.set_limits((lim1_tr,lim2_tr))
        
        # Reset pivot
        pivot_tr=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,mid)
        limits.set_pivot(pivot_tr)
        
        # Draw limits/pivot
        self.canvas.draw_axes()
        
    def is_mouse_over_line(self,event):
        threshold=4
        possible_match={}
        
        for line in self.canvas.lims:
            x0,x1=line.get_xdata()
            y0,y1=line.get_ydata()
            x0t,y0t= line.get_transform().xy_tup((x0, y0))
            x1t,y1t= line.get_transform().xy_tup((x1, y1))
            
            d = abs(x0t-event.x)
            
            if d<threshold and (event.y<y1t and event.y>y0t):
                possible_match[d]=line
            
        
        line=self.canvas.pivot
        
        if line is not None:
            x0,x1=line.get_xdata()
            y0,y1=line.get_ydata()
            x0t,y0t= line.get_transform().xy_tup((x0, y0))
            x1t,y1t= line.get_transform().xy_tup((x1, y1))
            
            d = abs(x0t-event.x)
            
            if d<threshold and event.y<y1t and event.y>y0t:
                possible_match[d]=line
        
         
        if len(possible_match.keys())>0:
            return possible_match[min(possible_match.keys())]
        else:
            return None
      
    def press(self, event):
        'this will be called whenever a mouse button is pressed'
        
        if self._active!= NMRFriend.NMRGRAPH_INTERACTION_MOVE_LINE:
            self._last_active=self._active
        
        if event.inaxes:
            self._movable_line=self.is_mouse_over_line(event)
            if self._movable_line is not None:
                self._active=NMRFriend.NMRGRAPH_INTERACTION_MOVE_LINE
            
        
        if self._active==NMRFriend.NMRGRAPH_INTERACTION_MOVE_LINE:
            self.press_move_line(self._movable_line,event)
            
        
        NMRFriend_Navigation_Toolbar.press(self, event)
    
    def release(self,event):
        
        NMRFriend_Navigation_Toolbar.release(self, event)
        
        if self._active==NMRFriend.NMRGRAPH_INTERACTION_MOVE_LINE:
            self.release_move_line(event)
            self._active=self._last_active
        
    def release_move_line(self, event):
        if self._idDrag is not None:
            self.canvas.mpl_disconnect(self._idDrag)
        
        # Store new position of line
        x1,x2=self._movable_line.get_xdata()
        x_tr=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,x1)
        
        # Find which line has been moved
        new_lims=[]
        for i,line in enumerate(self.canvas.lims):
            if line==self._movable_line:
                new_lims.append(x_tr)
            else:
                lim1,lim2=line.get_xdata()
                lim_tr=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,lim1)
                new_lims.append(lim_tr)
            
        
        if len(new_lims)==2:
            self.canvas.spectrum.limits.set_limits(tuple(new_lims))
        
        if self._movable_line==self.canvas.pivot:
            self.canvas.spectrum.limits.set_pivot(x_tr)
        
        
        # Reset flags
        self._idDrag=self.canvas.mpl_connect('motion_notify_event', self.mouse_move)
        self._movable_line=None
    
    def press_move_line(self,line,event):
        if self._idDrag is not None:
            self.canvas.mpl_disconnect(self._idDrag)
        
        self._idDrag=self.canvas.mpl_connect('motion_notify_event', self.drag_line)
    
    def drag_line(self,event):
        if self._lastCursor!=cursors.MOVE:
            self.set_cursor(cursors.MOVE)
                
            self._lastCursor = cursors.MOVE
        
        if event.inaxes and (self._movable_line is not None):
            old_x0,old_x1=self._movable_line.get_xdata()
            new_x,new_y=self._movable_line.get_transform().inverse_xy_tup((event.x, event.y))           
            self._movable_line.set_xdata([new_x,new_x])
            
            self.canvas.draw_idle()
        else:
            pass

    def moments_calc(self,*args):
        moments = self.canvas.spectrum.moments
        limits = self.canvas.spectrum.limits
        
        if moments is None:
            self.canvas.spectrum.moments = NMRFriend.NmrSpectrum.Moments(self.canvas.spectrum)
            moments = self.canvas.spectrum.moments
        
        if limits is None:
            GtkUtil.message_die("You must define limits BEFORE calculating moments")
            return
        
        # find lims from canvas
        if len(self.canvas.lims)>0:
            # get data values form lines
            lims1=self.canvas.lims[0].get_xdata()
            lims2=self.canvas.lims[1].get_xdata()
            
            # Converts limits from active units to points
            lim1=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,lims1[0])
            
            lim2=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,lims2[0])
            
            limits.set_limits((lim1,lim2))
        
        # Find pivot from canvas
        if self.canvas.pivot is not None:
            # get data values form line
            pivots=self.canvas.pivot.get_xdata()
            
            # Converts pivot from active units to points
            pivot=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,pivots[0])
            limits.set_pivot(pivot)
        
        # Calculate moments
        NMRFriend.Commands.calculate_moments()
        
        # Redraw display
        NMRFriend.main_app.main_view.rebuild()
    
    def depake(self,*args):
        limits = self.canvas.spectrum.limits
        
        if limits is None:
            GtkUtil.message_die("You must define limits BEFORE calculating moments")
            return
        
        # find lims from canvas
        if len(self.canvas.lims)>0:
            # get data values form lines
            lims1=self.canvas.lims[0].get_xdata()
            lims2=self.canvas.lims[1].get_xdata()
            
            # Converts limits from active units to points
            lim1=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,lims1[0])
            
            lim2=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,lims2[0])
            
            limits.set_limits((lim1,lim2))
        
        # Find pivot from canvas
        if self.canvas.pivot is not None:
            # get data values form line
            pivots=self.canvas.pivot.get_xdata()
            
            # Converts pivot from active units to points
            pivot=self.canvas.spectrum.old_units_to_new(self.canvas.active_units,NMRFriend.UNITS_PTS,pivots[0])
            limits.set_pivot(pivot)
        
        # Calculate moments
        NMRFriend.Commands.depake_spectrum()
        
        # Redraw display
        NMRFriend.main_app.main_view.rebuild()
        
    def toggle_units(self,*args):
        """Toggle units between points and seconds"""
        
        # get the new active units
        new_plot_x=self.canvas.spectrum.get_spc_axis(self.canvas.toggle_units())
        self.canvas.set_x_plot(new_plot_x)
        
        self.empty_views()

class Depaked_Navigation_Toolbar(NMRFriend_Navigation_Toolbar):
    # list of toolitems to add to the toolbar, format is:
    # text, tooltip_text, image_file, callback(str),button type,button group
    
    toolitems = NMRFriend_Navigation_Toolbar.toolitems + (('ToggleUnits', 'Toggle between Points, PPM and Hz units', 'units-spc.png', 'toggle_units',NMRFriend.TOOLBAR_BUTTON,None),
                                                          (None,None,None,None,None,None),
                                                          ('ExportDepaked','Export Depaked Spectrum to ASCII file','export-depaked.png','export_depaked',NMRFriend.TOOLBAR_BUTTON,'Shuffling'),
                                                          )
    
    def __init__(self, canvas, window):
        NMRFriend_Navigation_Toolbar.__init__(self,canvas,window)
        
    def toggle_units(self,*args):
        """Toggle units between points and seconds"""
        
        # get the new active units
        new_plot_x=self.canvas.spectrum.get_spc_axis(self.canvas.toggle_units())
        self.canvas.set_x_plot(new_plot_x)
        
        self.empty_views()
    
    def export_depaked(self, *args):
        NMRFriend.Commands.export_depaked()


class Fid_Navigation_Toolbar(NMRFriend_Navigation_Toolbar):
    
    # list of toolitems to add to the toolbar, format is:
    # text, tooltip_text, image_file, callback(str),button type,button group
    toolitems = NMRFriend_Navigation_Toolbar.toolitems + (('ToggleUnits', 'Toggle between Points and seconds units', 'units-fid.png', 'toggle_units',NMRFriend.TOOLBAR_BUTTON,None),
                                                          (None,None,None,None,None,None),
                                                          ('Shuffle','Shuffle Real and Imaginary FIDs','fid-shuf.png','fid_shuf',NMRFriend.TOOLBAR_RADIO_BUTTON,'Shuffling'),
                                                          ('UnshuffleHorz','Unshuffle Real and Imaginary FIDs horizontally','fid-horz-unshuf.png','fid_horz_unshuf',NMRFriend.TOOLBAR_RADIO_BUTTON,'Shuffling'),
                                                          ('UnshuffleVert','Unshuffle Real and Imaginary FIDs horizontally','fid-vert-unshuf.png','fid_vert_unshuf',NMRFriend.TOOLBAR_RADIO_BUTTON,'Shuffling'),
                                                          )
    def __init__(self, canvas, window):
        NMRFriend_Navigation_Toolbar.__init__(self,canvas,window)
        
    
    def toggle_units(self,*args):
        """Toggle units between points and seconds"""
        
        # get the new active units
        new_plot_x=self.canvas.spectrum.get_fid_axis(self.canvas.toggle_units())
        self.canvas.set_x_plot(new_plot_x)
        
        self.empty_views()
        

    
    def fid_shuf(self,*args):
        """Shuffle Fids"""
        
        NMRFriend.main_app.options.set_option("fid_unshuffling",NMRFriend.FID_UNSHUF_NONE)
        self.canvas.draw_axes()
        
        self.empty_views()
    
    def fid_horz_unshuf(self,*args):
        """Unshuffle Fids horizontally"""
        
        NMRFriend.main_app.options.set_option("fid_unshuffling",NMRFriend.FID_UNSHUF_HORZ)
        self.canvas.draw_axes()
        
        self.empty_views()
    
    
    def fid_vert_unshuf(self,*args):
        """Unshuffle Fids vertically"""
        
        NMRFriend.main_app.options.set_option("fid_unshuffling",NMRFriend.FID_UNSHUF_VERT)
        self.canvas.draw_axes()
        
        self.empty_views()

    