import os,sys,datetime

#NMR Friend imports
import NMRFriend
import NMRFriend.Toolbox.Util as Util

class OptionSet:
    """Internal class for option set handling."""
  
    def __init__(self,options_tuple=None):
        """Creates option set and add tuple of options (if not None) to the new built set."""
        self.options={}
    
        if options_tuple!=None:
            for option in options_tuple:
                self.add_option(option[0],option[1],option[2])
    
  
    def add_option(self,option_name,option_value,option_type):
        """Add option to option set"""
        option=Option(option_name,option_value,option_type)
        if self.options.has_key(option_name):
            raise IndexError,str(option_name)+" key already exists in "+str(self.options)
        else:
            self.options[option_name]=option
  
    def remove_option(self,option_name):
        """Remove option from option set."""
        if self.options.has_key(option_name):
            del self.options[option_name]
        else:
            raise IndexError,str(option_name)+" key does not exist in "+self.options
  
    def get_option(self,option_name):
        """Returns option value. Returns None if option_name is not correct."""
    
        if not self.options.has_key(option_name):
            return None
        else:
            option=self.options[option_name]
            return Util.str2num(option.get_value())
  
    def set_option(self,option_name,new_value):
        """Sets option."""
        if not self.options.has_key(option_name):
            raise IndexError,str(option_name)+" key does not exist in "+self.options
        else:
            option=self.options[option_name]
            option.set_value(Util.str2num(str(new_value)))
 
    def is_option_valid(self,option_name):
        """Returns True if option_name is name of a valid option.
        Otherwise return False.
        """
        return self.options.has_key(option_name)
  
    def get_options(self):
        """Returns dictionnary containing options names and values."""
        dict={}
        
        for key in self.options.keys():
            dict[key]=self.options[key].get_value()
        
        return dict
  

class Option:
    """Internal class for option"""
  
    def __init__(self,name,value,type):
        self.name=name
        self.value=value
        self.type=type
  
    def get_value(self):
        """Returns option value"""
        return self.value
  
    def set_value(self,new_value):
        """Sets new value to option"""
        self.value=new_value
    
    def get_name(self):
        """Returns option name"""
        return self.name
  
    def get_type(self):
        """Returns option type"""
        return self.type

def load_default_optionset():
    return OptionSet(NMRFriend.DEFAULT_OPTIONS)
