cdef extern from "math.h":
    double sqrt(double num)
    double log(double num)
    double fabs(double num)
    double pow(double base, double exp)

cdef extern from "Python.h":
    ctypedef struct PyThreadState:
        # this is a place holder
        pass
    PyThreadState* PyEval_SaveThread()
    void PyEval_RestoreThread(PyThreadState* state)

cdef double rat1(int m, int n):
    cdef int mp, mm
    cdef float z, rat
    mp = m + 1
    mm = m - 1
    z = mp * (mp + 2 * n)
    rat = sqrt(z) + mp + n
    z = mm * (mm + 2 * n)
    rat = rat / ( sqrt(z) + mm + n)
    z = fabs(rat)
    return log(z)

cdef double rat2(int n):
    cdef float z, rat
    z = 2 * fabs(n) + 1
    rat = sqrt(z) + n + 1
    if n < 0:
        z = n / (rat - 2.0)
    else:
        z = rat / n
    z = fabs (z)
    return log(z)

cdef double rat3(int n):
    cdef float z, rat, n_abs
    n_abs = fabs(n)
    z = (n_abs + 1.0) * ( 3.0 * n_abs + 1.)
    rat = sqrt(z) + 2 * n + 1
    if n < 0:
        z = -0.267949192 * n_abs / (rat - 2.0)
    else:
        z = rat / 3.732050808 / n_abs    
    z = fabs (z)
    return log(z)

def depake(spc, llim = 0, rlim = -1, n0 = -1, n_iter = 5):
    return depake_threaded(spc, llim, rlim, n0, n_iter)

cdef depake_threaded(spc_py, llim_py, rlim_py, n0_py, n_iter_py):
    cdef int iter,n_max, ncp, i, j, n, nn, i_hex, hex_flag, nsgnd, n0, n0n, n0r,  llim, rlim, n_iter, z
    cdef int n21, n22, jfin, m, m1
    cdef int n31, n32, n33, j1fin, j2fin
    cdef int i1, i2
    cdef double r[32768],y[32768], spc[32768], sum, orient
    cdef double ar1, ar2
    cdef double tmp
    
    cdef PyThreadState* state
    
    # Convert Python value to C
    llim = llim_py
    rlim= rlim_py
    n0 = n0_py
    n_iter = n_iter_py
    n_iter_d = n_iter
    
    oriented_spc = list()
    ncp = len(spc_py)
    
    
    
    if ncp > 32768:
        raise ValueError("Spectrum too big!")
    
    if rlim == -1:
        rlim = ncp - 1
    if n0 == -1:
        n0 = (rlim - llim) / 2
    
    n_max = 2 * int(max(abs(rlim - n0), abs(n0 - llim)) / 2. )
    
    #print "llim - n0 - rlim - n_max: %i - %i - %i - %i"%(llim,n0,rlim,n_max)
    hex_flag = -1 # hex_flag = 1 gives "edge on the left" (HEXagonal) configuration; hex_flag = -1 gives bilayer configuration
    hex_flag_d = hex_flag
    
    #print "hex_flag:", hex_flag
    for i from 0 <= i < ncp:
        r[i] = 0.0
        y[i] = 0.0
        spc[i] = spc_py[i]
    # Unlock GIL
    state = PyEval_SaveThread()
        
    for i from 1 <= i <= n_max:
    
        n = n_max - i + 1
        nn = 2 * n
        nsgnd = hex_flag * n           
        sum = 0.0
        
        if i == 1:
            n0r = n0 + 2 * nsgnd
            n0n = n0 - nsgnd
            orient = (1.732050808 * spc[n0n - 1] - sum ) / rat2(-nsgnd)
            #print "i,n0r,n0n,orient:",i,n0r, n0n, orient
            r[n0r - 1] = orient
            y[n0 + nsgnd - 1] = orient
            continue

        
        n21 = nn + 2
        n22 = 2 * n_max
        jfin = n22 - n21 + 1
        
        
        if hex_flag != 1:
            n21 = -2 * n_max
        
        j = 1
        while j <= jfin:
            m = n21 - 1 + j
            sum = sum + r[m + n0 - 1] * rat1(m, -nsgnd)
            #if i < 5:
                #print  "jfin loop: j, m, sum ->", j, m, sum
            
            j = j + 2
        
        n0r = n0 + 2 * nsgnd
        n0n = n0 - nsgnd
        orient = (1.732050808 * spc[n0n - 1] - sum ) / rat2(-nsgnd)
        
        r[n0r - 1] = orient
        y[n0 + nsgnd - 1] = orient
    
    for iter from 1 <= iter <= n_iter:
        if n_iter % 2 == 0:
            i_hex = hex_flag
        else:
            i_hex = -hex_flag

        for i from 1 <= i <= n_max:

           n = n_max + 1 - i
           nsgnd = i_hex * n
           
           ar1 = 0.0
           ar2 = 0.0
           
           
           if i == 1:
               n0r = n0 + 2 * nsgnd
               n0n = n0 - nsgnd
               orient = (1.732050808 * spc[n0n] - ar1 - ar2 - r[n0n] * rat3(-nsgnd) ) / rat2(-nsgnd)
               r[n0r] = orient
               y[n0 + nsgnd] = orient
               continue
           
           z = n / 2
           
           n31 = 2 * z + 2
           n32 = -2 * n_max
           n33 = -2 * n - 2
           
           if i_hex != -1:
               n31 = 2 * n +2
               n33 = -n -1
           
           j1fin = 2 * n_max - n31 + 1
           j2fin = n33 - n32 + 1
           
           j = 1
           while j <= j1fin:
               m = n31 - 1 + j
               ar1 = ar1 + r[m + n0 - 1] * rat1(m , -nsgnd)
               j = j + 2
           
           j = 1
           while j <= j2fin:
               m = n32 - 1 + j
               ar2 = ar2 + r[m + n0 - 1] * rat1(m , -nsgnd)
               j = j + 2
           
           n0r = n0 + 2 * nsgnd
           n0n = n0 - nsgnd
           orient = (1.732050808 * spc[n0n - 1] - ar1 - ar2 - r[n0n - 1] * rat3(-nsgnd) ) / rat2(-nsgnd)
           r[n0r - 1] = orient
           y[n0 + nsgnd - 1] = orient
    
    # Lock GIL
    PyEval_RestoreThread(state)
    
    for i from 0 <= i < n_max:
        i1 = n0 + i
        i2 = n0 - i
        temp = y[i2]
        y[i2] = y[i1]
        y[i1] = temp
        
    for i from 0 <= i < ncp:
        oriented_spc.append(y[i])
    return oriented_spc
