#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys,os
from distutils.core import setup
from distutils.extension import Extension
from distutils.command.install import install as _install
from distutils.command.build import build as _build

try:
    from Pyrex.Distutils import build_ext
    depaker_src = os.path.join("NMRFriend","Toolbox","Depaker.pyx")
except ImportError:
    print "Pyrex not found!"
    from distutils.command.build_ext import build_ext
    depaker_src = os.path.join("NMRFriend","Toolbox","Depaker.c")
    
    if not os.path.exists(depaker_src):
        print "%s not found! You have to install Pyrex before installing NMR-Friend"
        sys.exit(1)
    
INSTALLED_FILES = '.installed_files'

class build (_build):
    def run (self):
        _build.run(self)
        
        # Update version in Constant file
        fname = os.path.join (self.build_lib, "NMRFriend","Constants.py")
        f = open (fname, "rt")
        data = f.read ()
        f.close ()
        data = data.replace ("{Dev-Version}", version)
        f = open (fname, "wt")
        f.write (data)
        f.close ()

# Shamelessly stolen from ccsm and slightly corrected for NMR Friend
class install (_install):
    
    def run (self):

        _install.run(self)
        outputs = self.get_outputs()
        data = '\n'.join(outputs)
        try:
            f = open(INSTALLED_FILES, 'w')
        except:
            self.warn ('Could not write installed files list %s' %INSTALLED_FILES)
            return 

        f.write(data)
        f.close()
        
class uninstall(_install):
    description = "uninstall everything from installation directory"
    def run(self):
        try:
            files = file(INSTALLED_FILES, 'r').readlines()
        except:
            self.warn('Could not read installed files list %s' %INSTALLED_FILES)
            return

        for f in files:
            print 'Uninstalling %s' %f.strip()
            try:
                os.unlink(f.strip())
            except:
                self.warn('Could not remove file %s' %f)
        os.remove(INSTALLED_FILES)

# Load Version
version = open('VERSION', 'r').read().strip()

# Define packages to install
pixmap = [ os.path.join("Images", val) for val in os.listdir(os.path.join("NMRFriend","Images"))]

packages = ['NMRFriend','NMRFriend.Toolbox']
package_dir = {'NMRFriend' : 'NMRFriend'}
package_data = {'NMRFriend' : pixmap}

# Finally Install/Uninstall NMR Friend
setup(
    name='NMRFriend',
    version=version,
    description='NMR Friend is a little tool for Solid State NMR.',
    author='Sébastien Buchoux',
    author_email='seb.bubuntu@gmail.com',
    #url='http://opencompositing.org',
    packages=packages,
    package_dir = package_dir,
    package_data = package_data,
    scripts=['nmr-friend'],
    ext_modules=[
                 Extension("NMRFriend.Toolbox.Depaker", [depaker_src])
                 ],
    #data_files=data_files,
    cmdclass={
              'build' : build,
              'build_ext': build_ext,
              'uninstall': uninstall,
              'install': install
              },
)
